/*
 * Copyright 2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* Anonymous function is used not to pollute environment */
(function(Nuvola){
var _ = Nuvola.gettext;

/**
 * Object that manages a HTML form
 */
Nuvola.Form = Nuvola.makeClass(null, {
	/**
	 * Creates a new form widget from a form element with given id.
	 * 
	 * @param id	form element id
	 * @param reloadAction	action triggered from a reload notice, defaults to "refresh"
	 */
	$init: function(id, reloadAction){
		this.id = id;
		this.element = document.getElementById(id);
		this.table = Nuvola.makeElement("table");
		this.table.className =  "form";
		this.element.appendChild(this.table);
		this.children = [];
		this._notice = null;
		this.reloadAction = reloadAction || "refresh";
	},
	
	/**
	 * Appends a widget to the form
	 * 
	 * @param widget	widget to append
	 */
	append: function(widget){
		this.table.appendChild(widget.element);
		this.children.push(widget);
		widget.parent = this;
	},
	
	/**
	 * Prepends a widget to the form
	 * 
	 * @param widget	widget to prepend
	 */
	prepend: function(widget){
		if(this.table.firstChild){
			this.table.insertBefore(widget.element, this.table.firstChild);
			this.children.unshift(widget);
			widget.parent = this;
		}
		else{
			this.append(widget);
		}
	},
	
	/**
	 * Inserts a widget to the form
	 * 
	 * @param index		position in the form
	 * @param widget	widget to insert
	 */
	insert: function(index, widget){
		var children = this.table.childNodes;
		if(index < 0){
			index = Math.max(0, children.length -1);
		}
		if(index == 0){
			this.prepend(widget);
		}
		else if(index >= children.length){
			this.append(widget);
		}
		else{
			this.table.insertBefore(widget.element, children[index]);
			this.children.splice(index, 0, widget);
			widget.parent = this;
		}
	},
	
	/**
	 * Removes a widget from the form
	 * 
	 * @param widget	widget to remove
	 */
	remove: function(widget){
		this.table.removeChild(widget.element);
		this.children.splice(this.children.indexOf(widget), 1);
		widget.parent = null;
	},
	
	/**
	 * Shows a reload notice at the beginning of the form
	 */
	showReloadNotice: function(){
		if(!this._notice){
			this._notice = new Nuvola.ReloadNotice(this.reloadAction);
		}
		if(!this._notice.parent){
			this.prepend(this._notice);
		}
	}
});

Nuvola.Form._lastUid = 0;
Nuvola.Form.nextUid = function(){
	return "input" + (++Nuvola.Form._lastUid);
}

/**
 * A widget that can be added to a form.
 */
Nuvola.FormWidget = Nuvola.makeClass(null, {
	/**
	 * Creates a new form widget
	 */
	$init: function(){
		this.element = Nuvola.makeElement("tr");
		this.parent = null;
		this.timeout = 0;
	}
});

/**
 * A text entry that can be added to a form.
 */
Nuvola.Entry = Nuvola.makeClass(Nuvola.FormWidget, {
	/**
	 * Creates a new text entry
	 * 
	 * @param label		label shown before the entry
	 * @param value		value of the entry
	 * @param callback	a function called when entry is modified
	 */
	$init: function(label, value, callback){
		this.element = Nuvola.makeElement("tr");
		this._callback = callback;
		this.input = Nuvola.makeElement("input", {
			type: "text",
		});
		this.input.id = Nuvola.Form.nextUid();
		this.input.value = value;
		this.input.onchange = callback;
		this.input.onpaste = callback;
		this.input.oninput = this.input.onkeydown = (function(self, callback){
			return function(){
				if(self.timeout > 0){
					clearTimeout(self.timeout);
					self.timeout = 0;
				}
				self.timeout = setTimeout(Nuvola.bind(this, callback), 500);
			}
		})(this, callback);
		
		
		var cell = Nuvola.makeElement("td")
		label = Nuvola.makeElement("label", {"for": this.input.id}, label);
		cell.appendChild(label);
		this.element.appendChild(cell);
		cell = Nuvola.makeElement("td");
		cell.appendChild(this.input);
		this.element.appendChild(cell);
	}
});

/**
 * A checkbox that can be added to a form.
 */
Nuvola.Checkbox = Nuvola.makeClass(Nuvola.FormWidget, {
	/**
	 * Creates a new checkbox
	 * 
	 * @param label		label shown after the entry
	 * @param checked	a state of the checkbox
	 * @param callback	a function called when the checkbox is modified
	 */
	$init: function(label, checked, callback){
		this.element = Nuvola.makeElement("tr");
		var cell = Nuvola.makeElement("td", {colspan: 2});
		this.input = Nuvola.makeElement("input", {
			type: "checkbox",
		});
		if(checked){
			this.input.setAttribute("checked", true);
		}
		this.input.id = Nuvola.Form.nextUid();
		this.input.onchange = callback;
		cell.appendChild(this.input);
		cell.appendChild(Nuvola.makeElement("label", {"for": this.input.id}, label));
		this.element.appendChild(cell);
	}
});

/**
 * A box that can be added to a form.
 */
Nuvola.FormBox = Nuvola.makeClass(Nuvola.FormWidget, {
	/**
	 * Creates a new form box
	 * 
	 * @param className	CSS class of the box
	 */
	$init: function(className){
		this.element = Nuvola.makeElement("tr");
		this.box = Nuvola.makeElement("td", {colspan: 2});
		if(className){
			this.box.className = className;
		}
		this.element.appendChild(this.box);
	},
	
	/**
	 * Adds a DOM node to the box
	 * 
	 * @param child	child node
	 */
	add: function(child){
		this.box.appendChild(child);
	},
	
	/**
	 * Removes a DOM node from the box
	 * 
	 * @param child	child node
	 */
	remove: function(child){
		this.box.removeChild(child);
	}
});

/**
 * A reload notice that can be added to a form
 */
Nuvola.ReloadNotice = Nuvola.makeClass(Nuvola.FormBox, {
	/**
	 * Creates a new reload notice
	 * 
	 * @param action	action triggered when Reload button is clicked
	 */
	$init: function(action){
		this.$parent.$init.call(this, "notice");
		this.add(Nuvola.makeText(_("Please reload service to apply changes.") + " "));
		this.add(Nuvola.makeButton(_("Reload"), Nuvola.bind(this, function(){
			Nuvola.triggerAction(action);
			this.parent.remove(this);
		})));
	}
});

/**
 * A message that can be added to a form
 */
Nuvola.FormMessage = Nuvola.makeClass(Nuvola.FormBox, {
	/**
	 * Creates a new form message
	 * 
	 * @param message	text of the message
	 */
	$init: function(message){
		this.$parent.$init.call(this, "message");
		this.add(Nuvola.makeText(message));
	}
});

/**
 * Creates a HTML button.
 * 
 * @param label	text of the button
 * @param callback	a function triggered when button is clicked
 * @return	a HTML input type button element
 */
Nuvola.makeButton = function(label, callback){
	var elm = Nuvola.makeElement("input", {
		type: "button",
		value: label
	});
	elm.onclick = callback;
	return elm;
}


/**
 * Returns form manager for a form available in service settings page.
 * 
 * @return	a Nuvola.Form widget
 */
Nuvola.getSettingsForm = function(){
	return (new Nuvola.Form("settings"));
}
	
// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
