/*
 * Copyright 2011-2014 Jiří Janoušek <janousek.jiri@gmail.com>
 * Copyright 2014 Martin Pöhlmann <martin.deimos@gmx.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	var _ = Nuvola.gettext;
	
	// helper function for bind()
	var toArray = function(obj) {
		return Array.prototype.slice.call(obj);
	}
	
	/**
	 * Binds scope as "this" in the function fn
	 * Better JavaScript engines has this already
	 * implemented as fn.bind(scope).
	 */
	Nuvola.bind = function(scope, fn) {
	    return function () {
	        return fn.apply(scope, toArray(arguments));
	    };
	}
	
	/**
	 * Returns version of a library
	 * 
	 * @param string	library name
	 * @return array of major, minor and micro version
	 */
	Nuvola.libVersion = function(name)
	{
		var version = Nuvola.libs[name];
		if (!version)
			return undefined;
		
		version = version.split(".");
		for (var i = 0; i < version.length; i++)
			version[i] = 1*version[i];
		return version;
	}
	
	/**
	 * Triggers mouse event on element
	 * 
	 * @param elm Element object
	 * @param name Event name
	 */
	Nuvola.triggerMouseEvent = function(elm, name){
	    var event = document.createEvent('MouseEvents');
	    event.initMouseEvent(name, true, true, document.defaultView, 1, 0, 0, 0, 0, false, false, false, false, 0, elm);
	    elm.dispatchEvent(event);
	}

	/**
	 * Simulates click on element
	 * 
	 * @param elm Element object
	 */
	Nuvola.clickOnElement = function(elm){
	    Nuvola.triggerMouseEvent(elm, 'mouseover');
	    Nuvola.triggerMouseEvent(elm, 'mousedown');
	    Nuvola.triggerMouseEvent(elm, 'mouseup');
	    Nuvola.triggerMouseEvent(elm, 'click');
	}
	
	/**
	 * Creates HTML text node
	 * @param text	text of the node
	 * @return		new text node
	 */
	Nuvola.makeText = function(text){
		return document.createTextNode(text);
	}
	
	/**
	 * Creates HTML element
	 * @param name			element name
	 * @param attributes	element attributes (optional)
	 * @param text			text of the element (optional)
	 * @return				new HTML element
	 */
	Nuvola.makeElement = function(name, attributes, text){
		var elm = document.createElement(name);
		attributes = attributes || {};
		for(var key in attributes){
			elm.setAttribute(key, attributes[key]);
		}
		if(text !== undefined && text !== null){
			elm.appendChild(Nuvola.makeText(text));
		}
		return elm;
	}
	
	/**
	 * Make a JavaScript class.
	 * 
	 * @param parents	parent class or an array of object where the first one is a parent class and others are mixins [can be null]
	 * @param init		a function that initializes an object [can be null]
	 * @return			a constructor of the class
	 */
	Nuvola.makeClass = function(parents, methods){
		var constructor = methods.hasOwnProperty('$init') ? methods.$init : function(){};		
		var parent;
		if(parents){
			if(parents instanceof Array){
				parent = parents.length >= 1 ? parents.shift() : null;
			}
			else{
				parent = parents;
				parents = null;
			}
		}
		else{
			parent = null;
		}
		
		if(parent){
			// Create derived class
			var tmp = function(){};
			tmp.prototype = parent.prototype;
			constructor.prototype = new tmp();
		}
		else{
			parent = Object;
		}
		
		constructor.prototype.$parent = parent.prototype;
		
		if(parents){
			// Add mixins functions to the prototype
			var size = parents.length;
			for(var i = 0; i < size; i++){
				var mixin = parents[i];
				for(var key in mixin){
					var fn = mixin[key];
					if(!fn.name){
						fn.name = key;
					}
					constructor.prototype[key] = fn;
				}
			}
		}
		
		// Add methods
		for(var key in methods){
			var fn = methods[key];
			if(!fn.name){
				fn.name = key;
			}
			constructor.prototype[key] = fn;
		}
		
		return constructor;
	}
	
	Nuvola.checkFlash = function(formats)
	{
		var formats = formats || Nuvola.formatSupport();
		return Nuvola.checkFlashAvailable(formats) && Nuvola.checkFlashEnabled(formats);
	}
	
	Nuvola.checkFlashAvailable = function(formats)
	{
		var formats = formats || Nuvola.formatSupport();
		if (Nuvola.config.checkFlashAvailable !== false && !formats.flash_available)
		{
			var show_again = Nuvola.warn("Flash plugin is not installed",
			"This web-based streaming services utilizes Adobe Flash for music playback, but no compatible Flash plugin has been found. "
			+ "Nuvola Player requires Flash plugin wrapped with nspluginwrapper because of conflicting libraries (GTK+ 2 and GTK+ 3)."
			+ "\n\n<a href=\"http://nuvolaplayer.fenryxo.cz/help/issues/flash_plugin.html\">Learn more about the issue and how to install compatible Flash plugin</a>");
			if (show_again === false)
			{
				Nuvola.config.checkFlashAvailable = show_again;
				Nuvola.saveConfig();
			}
		}
		return formats.flash_available;
	}
	
	Nuvola.checkFlashEnabled = function(formats)
	{
		var formats = formats || Nuvola.formatSupport();
		if (Nuvola.config.checkFlashEnabled !== false && !formats.flash_enabled)
		{
			var show_again =  Nuvola.warn("Flash plugin is not enabled",
			"This web-based streaming services utilizes Adobe Flash for music playback, but Flash plugin is disabled in service's settings."
			+ " The streaming service might not be able to play music unless you enable the Flash plugin.\n\n" +
			"<b>You can read more details about playback requirements for this service in the service selector dialog:</b>"
			+ " menu Application → Switch service.");
			
			if (show_again === false)
			{
				Nuvola.config.checkFlashEnabled = show_again;
				Nuvola.saveConfig();
			}
		}
		return formats.flash_enabled;
	}
	
	Nuvola.checkMp3 = function(formats)
	{
		var formats = formats || Nuvola.formatSupport();
		return Nuvola.checkMp3Enabled(formats) && Nuvola.checkMp3Available(formats);
	}
	
	Nuvola.checkMp3Available = function(formats)
	{
		var formats = formats || Nuvola.formatSupport();
		if (Nuvola.config.checkMp3Available !== false && !formats.mp3_available)
		{
			var show_again = Nuvola.warn("HTML5 Audio with MP3 support is not installed",
			"This web-based streaming services can utilize HTML5 Audio technology for music playback instead of Flash plugin, but no compatible GStreamer audio library has been found. "
			+ "Nuvola Player requires GStreamer with MP3 decoder plugin to provide HTML5 Audio support."
			+ "\n\n<a href=\"http://nuvolaplayer.fenryxo.cz/help/issues/html5_audio.html\">Learn more about the issue and how to install HTML5 Audio support</a>");
			if (show_again === false)
			{
				Nuvola.config.checkMp3Available = show_again;
				Nuvola.saveConfig();
			}
		}
		return formats.mp3_available;
	}
	
	Nuvola.checkMp3Enabled = function(formats)
	{
		var formats = formats || Nuvola.formatSupport();
		if (Nuvola.config.checkMp3Enabled !== false && !formats.mp3_enabled)
		{
			var show_again =  Nuvola.warn("HTML5 Audio library is not enabled",
			"This web-based streaming service can utilize HTML5 Audio technology for music playback instead of Flash plugin, but GStreamer audio library providing that feature has been disabled in preferences."
			+ " The streaming service might not be able to play music unless you enable the GStreamer audio library.\n\n" +
			"<b>You can read more details about playback requirements for this service in the service selector dialog:</b>"
			+ " menu Application → Switch service.");
			
			if (show_again === false)
			{
				Nuvola.config.checkMp3Enabled = show_again;
				Nuvola.saveConfig();
			}
		}
		return formats.mp3_enabled;
	}
	
	/**
	 * Add getter/setter for object's property.
	 * 
	 * @param klass    class constructor
	 * @param name     property name
	 */
	Nuvola.addProperty = function(klass, name)
	{
		Object.defineProperty(klass.prototype, name, {
			get: function() { return this.prop(name); },
			set: function(val) { this.prop(name, val); }
		});
	}
	
	/**
	 * Adds CSS style sheet
	 * 
	 * @since                 Nuvola JS API 2.3
	 * @param css [string]    CSS styles to apply
	 * @return style element that has been applied
	 */
	Nuvola.addStyleSheet = function(css)
	{
		var head = document.getElementsByTagName('head')[0];
		var style = document.createElement('style');
		style.type = 'text/css';
		if (style.styleSheet)
			style.styleSheet.cssText = css;
		else
			style.appendChild(document.createTextNode(css));
		head.appendChild(style);
		return style;
	}
	
	/**
	 * Removes CSS style sheet
	 * 
	 * @since          Nuvola JS API 2.3
	 * @param style    style element to remove
	 * @return true on success, false otherwise
	 */
	Nuvola.removeStyleSheet = function(style)
	{
		var parent = style.parentNode;
		if (parent)
		{
			parent.removeChild(style);
			return true;
		}
		return false;
	}
	
	/**
	 * class Nuvola.Action
	 * 
	 * Wrapper object for accessing and triggering GTKActions
	 *
	 * @since          Nuvola JS API 2.3
	 */
	{
		/**
		 * Creates new Action object.
		 * 
		 * @param name   action name [string]
		 */
		Nuvola.Action = function(name)
		{
			this.name = name;
		}
	
		/**
		 * Manipulates with action property
		 * 
		 * @param name     action name [string]
		 * @param value    undefined to get the current value or a new value to set the new value
		 */
		Nuvola.Action.prototype.prop = function(name, value)
		{
			if (value === undefined)
				return Nuvola.getActionProperty(this.name, name);
		
			if (name != "sensitive")
				throw new Error("Property '" + name + "' is read-only.");
		
			Nuvola.updateAction(this.name, value);
		}
	
		/**
		 * Triggers action
		 */
		Nuvola.Action.prototype.trigger = function()
		{
			return Nuvola.triggerAction(this.name);
		}
	
		/**
		 * @property Nuvola.Action.sensitive   whether the action is enabled [bool]
		 */
		Nuvola.addProperty(Nuvola.Action, "sensitive");
	}
	
	/**
	 * class Nuvola.NavigationButtonIntegration
	 * 
	 * Support object for integrating and handling navigation buttons.
	 * This class requires that Nuvola.onMessageReceived is not overridden by the service integration. 
	 *
	 * @since          Nuvola JS API 2.3
	 */
	{
		/**
		 * Creates a new NavigationButtonIntegration instance.
		 *
		 * @param backButton      the back button [Element]
		 * @param forwardButton   the forward button [Element]
		 */
		Nuvola.NavigationButtonIntegration = function(backButton, forwardButton)
		{
			this.backButton = backButton;
			this.backAction = new Nuvola.Action(Nuvola.NavigationButtonIntegration.BACK);
			this.backButtonDisplayEnabled = this.backButton.style.display;
			this.setSensitivity(this.backButton, this.backAction);
			this.backButton.addEventListener('click', this.backAction.trigger.bind(this.backAction));
			
			this.forwardButton = forwardButton;
			this.forwardAction = new Nuvola.Action(Nuvola.NavigationButtonIntegration.FORWARD);
			this.forwardButtonDisplayEnabled = this.forwardButton.style.display;
			this.setSensitivity(this.forwardButton, this.forwardAction);
			this.forwardButton.addEventListener('click', this.forwardAction.trigger.bind(this.forwardAction));
		}
		
		/** @private Prefix for navigation back button and back action. */
		Nuvola.NavigationButtonIntegration.BACK = "back";
	
		/** @private Prefix for navigation forward button and action. */
		Nuvola.NavigationButtonIntegration.FORWARD = "forward";
		
		/** Enables or disables the buttons by toggling style.display 'hidden'. */
		Nuvola.NavigationButtonIntegration.prototype.setEnabled = function(enabled)
		{
			if (enabled)
			{
				this.backButton.style.display = this.backButtonDisplayEnabled;
				this.forwardButton.style.display = this.forwardButtonDisplayEnabled;
			}
			else
			{
				this.backButton.style.display = 'none';
				this.forwardButton.style.display = 'none';
			}
		}
		
		/** Sets the enabled state of a navigation button according to the action sensitivity. Override for custom behavior. */
		Nuvola.NavigationButtonIntegration.prototype.setSensitivity = function(button, action)
		{
			button.disabled = !action.sensitive;
		}
		
		/** Handles action changes to adjust the sensitivity state of the buttons. Returns true if the message was handled. */
		Nuvola.NavigationButtonIntegration.prototype.onMessageReceived = function(message, action, property)
		{
			if (message !== Nuvola.ACTION_CHANGED && property !== "sensitive")
			{
				return false;
			}
			
			var button = this[action + "Button"];
			var action = this[action + "Action"];
			
			if (button === undefined || action === undefined)
			{
				return false;
			}
			
			this.setSensitivity(button, action);
			return true;
		}
	}

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
