/*
 * Copyright 2011-2014 Jiří Janoušek <janousek.jiri@gmail.com>
 * Copyright 2014 Martin Pöhlmann <martin.deimos@gmx.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	var ACTIVITY_INTERVAL = 15*60*1000;
	
	if (Nuvola.warn && Nuvola.config.warnDesktopNotifications !== false)
	{
		var show_again = Nuvola.warn("Google Play Music Compatibility Issue",
		"Google Play Music loads only search bar and sidebar and then hangs when desktop notifications are enabled in Google Play Music Labs settings."
		+ " As a workaround, you can disable Desktop notifications in Labs settings using Firefox, Chromium or Google Chrome."
		+ "\n\n<a href=\"https://bugs.launchpad.net/nuvola-player/+bug/1314753\">Learn more about the issue and how to apply the workaround</a>."
		+ "\n\n<i>This issue doesn't affect desktop notifications provided by Nuvola Player, only built-in Google Play Music desktop notifications for Chrome or Chromium browsers.</i>" );
		if (show_again === false)
		{
			Nuvola.config.warnDesktopNotifications = show_again;
			Nuvola.saveConfig();
		}
	}
	
	if (Nuvola.libVersion && Nuvola.warn)
	{
		var gst = Nuvola.libVersion("gstreamer");
		if (gst[0] == 1 && gst[1] == 0 && Nuvola.config.warnGstreamer1dotx !== false)
		{
			var show_again = Nuvola.warn("Problematic library detected: GStreamer " + Nuvola.libs.gstreamer,
			"Nuvola Player utilizes GStreamer library to provide support for HTML5 Audio technology used by some streaming services for music playback."
			+ "\n\nUnfortunately, GStreamer library available on your system (version " + Nuvola.libs.gstreamer + ") is known to have a bug causing malfunction of Google Play Music in Nuvola Player."
			+ " Hopefully, the bug will be fixed in the next version of GStreamer library. <b>Meanwhile, you can apply a workaround. It is not perfect and has some side effects, so it is not enabled by default for that reason.</b>"
			+ "\n\n<a href=\"http://nuvolaplayer.fenryxo.cz/help/issues/gstreamer_1.0.x.html\">Learn more about the issue and how to apply the workaround</a>." );
			if (show_again === false)
			{
				Nuvola.config.warnGstreamer1dotx = show_again;
				Nuvola.saveConfig();
			}
			Nuvola.checkFlash();
		}
		else
		{
			Nuvola.checkFlash() || Nuvola.checkMp3();
		}
	}
	
	var hasClass = function(elm, className)
	{
		var classes = elm.className.split(" ");
		var i = classes.length;
		while (i--)
		{
			if (classes[i] === className)
				return true;
		}
		
		return false;
	}
	
	/**
	 * Creates Google Music integration binded to Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
		
		/* For debug output */
		this.name = "Google Music";		
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.can_thumbs_up = null;
		this.can_thumbs_down = null;
		this.can_prev = null;
		this.can_next = null;
		this.update();
		this.applyExtraFeatures();
	};
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		// Default state
		var state = Nuvola.STATE_NONE;
		var can_prev;
		var can_next;
		var can_thumbs_up;
		var can_thumbs_down;
			
		try
		{
			var album_art = document.getElementById('playingAlbumArt').src;
		}
		catch(e)
		{
			var album_art = null;
		}
		
		try
		{
			var song = document.getElementById('playerSongTitle').firstChild;
			song = song.innerText || song.textContent;
		}
		catch(e)
		{
			var song = null;
		}
		
		var artistDiv = document.getElementById('player-artist');
		try
		{
			var artist = artistDiv.firstChild;
			artist = artist.innerText || artist.textContent;
		}
		catch (e)
		{
			var artist = null;
		}
		
		try
		{
			var album = artistDiv.nextSibling.nextSibling.firstChild;
			album = album.innerText || album.textContent;
		}
		catch (e)
		{
			var album = null;
		}
		
		try
		{
			var buttons = this.getPlayerButtons();
			var play_pause = buttons[2];
			if (play_pause.disabled === true)
				state = Nuvola.STATE_NONE;
			else if (play_pause.className == "flat-button playing")
				state = Nuvola.STATE_PLAYING;
			else
				state = Nuvola.STATE_PAUSED;
			
			if (state != Nuvola.STATE_NONE)
			{
				can_prev = buttons[1].disabled === false;
				can_next = buttons[3].disabled === false;
			}
			else
			{
				can_prev = can_next = false;
			}
			
		}
		catch (e)
		{
			can_prev = can_next = false;
		}
		
		try
		{
			var thumbs = this.getThumbs();
			
			if (thumbs[0].style.visibility == "hidden")
			{
				can_thumbs_down = can_thumbs_up = false;
			}
			else
			{
				can_thumbs_up = thumbs[1].className != "selected";
				can_thumbs_down = thumbs[2].className != "selected";
			}
		}
		catch (e)
		{
			can_thumbs_down = can_thumbs_up = false;
		}
		
		this.state = state;
		
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, album, album_art, state);
		
		if(this.can_prev !== can_prev){
			this.can_prev = can_prev;
			Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, can_prev);
		}
		if(this.can_next !== can_next){
			this.can_next = can_next;
			Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);
		}
		if(this.can_thumbs_up !== can_thumbs_up){
			this.can_thumbs_up = can_thumbs_up;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_UP, can_thumbs_up);
		}
		if(this.can_thumbs_down !== can_thumbs_down){
			this.can_thumbs_down = can_thumbs_down;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_DOWN, can_thumbs_down);
		}
		
		setTimeout(Nuvola.bind(this, this.update), 500);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd, param1, param2)
	{
		if (cmd === Nuvola.CONFIG_CHANGED)
		{
			Nuvola.reloadConfig();
			this.applyExtraFeatures();
			return;
		}
		
		// Return if navigation buttons are enabled and can handle this event.
		if (this.navigationButtons !== undefined && this.navigationButtons.onMessageReceived(cmd, param1, param2) === true)
		{
			return;
		}
		
		/* SJBpost is an internal Google Music JavaScript function */
		var buttons = this.getPlayerButtons();
		if (buttons)
		{
			var prev_song = buttons[1];
			var next_song = buttons[3];
			var play_pause = buttons[2];
		}
		else
		{
			var prev_song = null;
			var next_song = null;
			var play_pause = null;
		}
		
		try{
			switch(cmd){
			case Nuvola.ACTION_PLAY:
				if(this.state != Nuvola.STATE_PLAYING)
					Nuvola.clickOnElement(play_pause);
				break;
			case Nuvola.ACTION_PAUSE:
				if(this.state == Nuvola.STATE_PLAYING)
					Nuvola.clickOnElement(play_pause);
				break;
			case Nuvola.ACTION_TOGGLE_PLAY:
				Nuvola.clickOnElement(play_pause);
				break;
			case Nuvola.ACTION_PREV_SONG:
				if (prev_song)
					Nuvola.clickOnElement(prev_song);
				break;
			case Nuvola.ACTION_NEXT_SONG:
				if (next_song)
					Nuvola.clickOnElement(next_song);
				break;
			case Nuvola.ACTION_THUMBS_UP:
				// Emulate click on thumbs up button
				Nuvola.clickOnElement(this.getThumbs()[1]);
				break;
			case Nuvola.ACTION_THUMBS_DOWN:
				// Emulate click on thumbs down button
				Nuvola.clickOnElement(this.getThumbs()[2]);
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' handled.");
		}
		catch(e){
			// API expects exception to be a string!
			throw (this.name + ": " + e.message);
		}
	}
	
	Integration.prototype.applyExtraFeatures = function()
	{
		/* Loading in progress? */
		var loading = document.getElementById("loading-progress");
		if (loading && loading.style.display != "none")
		{
			setTimeout(Nuvola.bind(this, this.applyExtraFeatures), 250);
			return;
		}
		
		if (Nuvola.config.simulateActivity === true && this.simulateActivityTimeout === undefined)
		{
			var callback = Nuvola.bind(this, this.simulateActivity);
			this.simulateActivityTimeout = setTimeout(callback, ACTIVITY_INTERVAL);
		}
		else if (Nuvola.config.simulateActivity !== true && this.simulateActivityTimeout !== undefined)
		{
			window.clearTimeout(this.simulateActivityTimeout);
			this.simulateActivityTimeout = undefined;
		}

		this.integrateNavigationButtons();
	}
	
	Integration.prototype.simulateActivity = function(){
		var elm = document.getElementById("insideBox");
		if(elm){
			console.log("Click on " + elm);
			Nuvola.clickOnElement(elm);
			setTimeout(Nuvola.bind(this, this.simulateActivity), ACTIVITY_INTERVAL);
		}
	}
	
	Integration.prototype.integrateNavigationButtons = function(){
		// This feature is disabled on pre 2.4.0
		if (Nuvola.NavigationButtonIntegration === undefined)
		{
			return;
		}

		// If the navigation buttons are already integrated, adjust enablement state.
		if (this.navigationButtons !== undefined)
		{
			this.navigationButtons.setEnabled(Nuvola.config.navigationButtons !== false);
			return;
		}
		
		// Otherwise, if not yet integrated and disabled, do nothing.
		if (Nuvola.config.navigationButtons === false)
		{
			return;
		}
		
		var queryBar = document.getElementById("gbq2");
		if (!queryBar)
		{
			console.log("Could not find the query bar.");
			return;
		}
		
		var queryBarFirstChild = queryBar.firstChild;
		
		var navigateBack = Nuvola.makeElement("button", null, "<");
		navigateBack.className = "button small vertical-align";
		navigateBack.style.float = "left";
		navigateBack.style.marginRight = "0px";
		navigateBack.style.borderTopRightRadius = "2px";
		navigateBack.style.borderBottomRightRadius = "2px";
		queryBar.insertBefore(navigateBack, queryBarFirstChild);
		
		var navigateForward = Nuvola.makeElement("button", null, ">");
		navigateForward.className = "button small vertical-align";
		navigateForward.style.float = "left";
		navigateForward.style.marginRight = "15px";
		navigateForward.style.borderLeft = "none";
		navigateForward.style.borderTopLeftRadius = "2px";
		navigateForward.style.borderLeftRightRadius = "2px";
		queryBar.insertBefore(navigateForward, queryBarFirstChild);
		
		// If webkit browser does not support flexible boxes, we have to add a right margin
		// to the right of the query bar with width of the buttons (incl. margin)
		if (!("flex" in document.body.style || "-webkit-flex" in document.body.style))
		{
			var buttonWidth = navigateBack.offsetWidth
				+ navigateForward.offsetWidth 
				+ parseInt(navigateForward.style.marginRight);
			queryBarFirstChild.style.marginRight = buttonWidth + "px";

			window.dispatchEvent(new Event('resize')); // force update in google script
		}

		this.navigationButtons = new Nuvola.NavigationButtonIntegration(navigateBack, navigateForward);
	}
	
	
	Integration.prototype.getPlayerButtons = function()
	{
		var elm = document.querySelector("#player .player-middle");
		return elm ? elm.childNodes : null;
	}
	
	Integration.prototype.getThumbs = function()
	{
		var elm = document.getElementsByClassName("thumbs")[0];
		return [elm, elm.childNodes[0], elm.childNodes[1]];
	}
	
	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);

// Force open pop-up targets in the main window, so we can apply sandbox on them too.
window.open = function(uri){
	console.log("Opening a pop-up window target in the main frame: " + uri);
	window.location.href = uri;
}
