/*
 * Copyright 2013 Michael Mims <mims.michael@gmail.com>
 * Copyright 2014 Martin Pöhlmann <martin.deimos@gmx.de>
 * Copyright 2014 Jiří Janoušek <janousek.jiri@gmail.com>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
	if (Nuvola.checkFlash)
		Nuvola.checkFlash();
	
	/**
	 * Creates Rdio integration binded to Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default command function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
		
		/* For debug output */
		this.name = "Rdio";
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.firstPlayToggled = false;
		this.update();
		this.integrateNavigationButtons();
	};

	/**
	 * Toggles playback the first time a play action is received
	 * This is a temporary hack to try to work around a known issue with WebkitGTK
	 * initial volume levels (although testing seems to indicate it might be on rdio
	 * client js side). See Nuvola bug LP:1049253.
	 */
	Integration.prototype.firstPlayToggle = function () {
		if(this.firstPlayToggled != true) {
			try {
				var player = R.Services.Player;
				player.playPause();
				
				setTimeout(function () {
					player.playPause();
					console.debug("[firstPlayToggle] First play toggle completed");
				}, 750);

				console.debug("[firstPlayToggle] First play toggle began");
				this.firstPlayToggled = true;
			}
			catch(e) {
				console.debug("[firstPlayToggle] Player service not available");
			}
		}
	}

	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		// Default state
		var state = Nuvola.STATE_NONE;
		var album = null;
		var album_art = null;
		var artist = null;
		var song = null;
		var can_prev;
		var can_next;

		try{
			var playingTrack = R.Services.Player.model.get("playingTrack").attributes;
			album_art = playingTrack.icon;
			album = playingTrack.album;
			artist = playingTrack.artist;
			song = playingTrack.name;
		}
		catch(e){
			console.debug("Unable to obtain song info: " + e.message);
		}
		
		try{
			var state = Nuvola.STATE_NONE;

			var rdiostate = R.Services.Player.model.attributes.playState;

			switch (rdiostate){
			case 0:
				state = Nuvola.STATE_PAUSED;
				break;
			case 1:
				state = Nuvola.STATE_PLAYING;
				this.firstPlayToggle();
				break;
			}
			
			if(state != Nuvola.STATE_NONE){
				can_prev = can_next = true;
			}
			else{
				can_prev = can_next = false;
			}
			
		}
		catch(e){
			// console.debug("Unable to obtain state info: " + e.message);
			can_prev = can_next = false;
		}

		this.state = state;
		
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, album, album_art, state);

		if(this.can_next !== can_next){
			this.can_next = can_next;
			Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);
		}
		if(this.can_prev !== can_prev){
			this.can_prev = can_prev;
			Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, can_prev);
		}
		
		setTimeout(Nuvola.bind(this, this.update), 500);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd, param1, param2){
		// Return if navigation buttons are enabled and can handle this event.
		if (this.navigationButtons !== undefined 
		&& this.navigationButtons.onMessageReceived(cmd, param1, param2) === true)
			return;
		
		try{
			switch(cmd){
			case Nuvola.ACTION_PLAY:
				if(this.state != Nuvola.STATE_PLAYING) R.Services.Player.playPause();
				break;
			case Nuvola.ACTION_PAUSE:
				if(this.state == Nuvola.STATE_PLAYING) R.Services.Player.playPause();
				break;
			case Nuvola.ACTION_TOGGLE_PLAY:
				R.Services.Player.playPause();
				break;
			case Nuvola.ACTION_PREV_SONG:
				R.Services.Player.previous();
				break;
			case Nuvola.ACTION_NEXT_SONG:
				R.Services.Player.next();
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string!
			throw (this.name + ": " + e.message);
		}
	}
	
	Integration.prototype.getSearchContainer = function()
	{
		return document.querySelector("#header .search_container");
	}
	
	Integration.prototype.integrateNavigationButtons = function()
	{
		// This feature is disabled on pre 2.4.0
		if (Nuvola.NavigationButtonIntegration === undefined)
			return;
		
		if (!this.getSearchContainer())
		{
			console.log("Could not find the search box.");
			setTimeout(Nuvola.bind(this, this.integrateNavigationButtons), 500);
			return;
		}
		
		var marginTop = "20px";
		var navigateBack = this.navigateBack = Nuvola.makeElement("button", null, "<");
		navigateBack.className = "button";
		navigateBack.style.float = "left";
		navigateBack.style.marginRight = "0px";
		navigateBack.style.marginTop = marginTop;
		navigateBack.style.borderTopRightRadius = "0px";
		navigateBack.style.borderBottomRightRadius = "0px";
		navigateBack.style.verticalAlign = "middle";
		
		var navigateForward = this. navigateForward = Nuvola.makeElement("button", null, ">");
		navigateForward.className = "button";
		navigateForward.style.float = "left";
		navigateForward.style.marginRight = "15px";
		navigateForward.style.marginTop = marginTop;
		navigateForward.style.borderLeft = "none";
		navigateForward.style.borderTopLeftRadius = "0px";
		navigateForward.style.borderBottomLeftRadius = "0px";
		
		this.insertNavigationButtons();
		this.navigationButtons = new Nuvola.NavigationButtonIntegration(navigateBack, navigateForward);
		
		// The #header element is occasionally removed from the DOM tree,
		// so we need to re-insert our navigation buttons.
		$("body").bind("DOMNodeRemoved", Nuvola.bind(this, function(e)
		{
			if (e.target.id == "header")
				setTimeout(Nuvola.bind(this, this.insertNavigationButtons), 100);
		}));
	}
	
	Integration.prototype.insertNavigationButtons = function()
	{
		var cursor = this.getSearchContainer();
		if (!cursor)
		{
			setTimeout(Nuvola.bind(this, this.insertNavigationButtons), 500);
			return;
		}
		
		var container = cursor.parentNode;
		container.insertBefore(this.navigateBack, cursor);
		container.insertBefore(this.navigateForward, cursor);
	}

	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton
	
// Call anonymous function with JS API object as an argument
})(this);
